@@ -1,4 +1,5 @@
-
+#ifndef BINARYSEARCHARRAY_H
+#define BINARYSEARCHARRAY_H
 
 
 struct key_val_tuple {
@@ -18,3 +19,4 @@ osmid_t binary_search_get(struct binary_search_array * array, int key);
 struct binary_search_array *  init_search_array(int capacity);
 void shutdown_search_array(struct binary_search_array ** array);
 
+#endif
