@@ -237,6 +237,7 @@ static void stop_error_copy(void)
 static int split_tags(struct keyval *tags, unsigned int flags, struct keyval *names, struct keyval *places, struct keyval *extratags, 
    int* admin_level, struct keyval ** housenumber, struct keyval ** street, struct keyval ** addr_place, char ** isin, struct keyval ** postcode, struct keyval ** countrycode)
 {
+   size_t subval;
    int placehouse = 0;
    int placebuilding = 0;
    int placeadmin = 0;
@@ -299,6 +300,7 @@ static int split_tags(struct keyval *tags, unsigned int flags, struct keyval *na
           strcmp(item->key, "old_name") == 0 ||
           (strncmp(item->key, "old_name:", 9) == 0) || 
           strcmp(item->key, "alt_name") == 0 ||
+          (strncmp(item->key, "alt_name_", 9) == 0) || 
           (strncmp(item->key, "alt_name:", 9) == 0) || 
           strcmp(item->key, "official_name") == 0 ||
           (strncmp(item->key, "official_name:", 14) == 0) || 
@@ -362,8 +364,7 @@ static int split_tags(struct keyval *tags, unsigned int flags, struct keyval *na
                strcmp(item->key, "office") == 0 ||
                strcmp(item->key, "railway") == 0 ||
                strcmp(item->key, "shop") == 0 ||
-               strcmp(item->key, "tunnel") == 0 ||
-               strcmp(item->key, "waterway") == 0 )
+               strcmp(item->key, "tunnel") == 0 )
       {
          if (strcmp(item->value, "no"))
          {
@@ -378,6 +379,11 @@ static int split_tags(struct keyval *tags, unsigned int flags, struct keyval *na
             freeItem(item);
          }
       }
+      else if (strcmp(item->key, "waterway") == 0 &&
+               strcmp(item->value, "riverbank") != 0)
+      {
+            pushItem(places, item);
+      }
       else if (strcmp(item->key, "place") == 0) 
       {
          place = item;
@@ -467,11 +473,24 @@ static int split_tags(struct keyval *tags, unsigned int flags, struct keyval *na
              addItem(places, "place", "houses", 1);
           }
       }
+      else if (strcmp(item->key, "tiger:county") == 0)
+      {
+         /* strip the state and replace it with a county suffix to ensure that
+            the tag only matches against counties and not against some town
+            with the same name.
+          */
+         subval = strcspn(item->value, ",");
+         *isin = realloc(*isin, isinsize + 9 + subval);
+         *(*isin+isinsize) = ',';
+         strncpy(*isin+1+isinsize, item->value, subval);
+         strcpy(*isin+1+isinsize+subval, " county");
+         isinsize += 8 + subval;
+         freeItem(item);
+      }
       else if (strcmp(item->key, "is_in") == 0 ||
           (strncmp(item->key, "is_in:", 5) == 0) ||
-          strcmp(item->key, "addr:country")== 0 ||
+          strcmp(item->key, "addr:suburb")== 0 ||
           strcmp(item->key, "addr:county")== 0 ||
-          strcmp(item->key, "tiger:county")== 0 ||
           strcmp(item->key, "addr:city") == 0 ||
           strcmp(item->key, "addr:state_code") == 0 ||
           strcmp(item->key, "addr:state") == 0)
@@ -642,13 +661,13 @@ static int split_tags(struct keyval *tags, unsigned int flags, struct keyval *na
 
    if (landuse)
    {
-      if (!listHasData(places))
+      if (!listHasData(places) && listHasData(names))
       {
           pushItem(places, landuse);
       }
       else
       {
-          freeItem(item);
+          freeItem(landuse);
       }
    }
 
@@ -1221,6 +1240,7 @@ static int gazetteer_process_relation(osmid_t id, struct member *members, int me
    struct keyval * countrycode;
    int wkt_size;
    const char *type;
+   int cmp_waterway;
 
    type = getItem(tags, "type");
    if (!type) {
@@ -1228,14 +1248,16 @@ static int gazetteer_process_relation(osmid_t id, struct member *members, int me
       return 0;
    }
 
-   if (!strcmp(type, "associatedStreet") || !strcmp(type, "relatedStreet"))
+   cmp_waterway = strcmp(type, "waterway");
+
+   if (!strcmp(type, "associatedStreet"))
    {
       Options->mid->relations_set(id, members, member_count, tags);
       if (delete_old) delete_unused_classes('R', id, 0); 
       return 0;
    }
 
-   if (strcmp(type, "boundary") && strcmp(type, "multipolygon")) {
+   if (strcmp(type, "boundary") && strcmp(type, "multipolygon") && cmp_waterway) {
       if (delete_old) delete_unused_classes('R', id, 0); 
       return 0;
    }
@@ -1268,29 +1290,53 @@ static int gazetteer_process_relation(osmid_t id, struct member *members, int me
          count++;
       }
 
+      if (count == 0)
+      {
+          if (delete_old) delete_unused_classes('R', id, 0);
+          free(xcount);
+          free(xtags);
+          free(xnodes);
+          free(xid2);
+          return 0;
+      }
+
       count = Options->mid->ways_get_list(xid2, count, &xid, xtags, xnodes, xcount);
 
       xnodes[count] = NULL;
       xcount[count] = 0;
 
-      wkt_size = build_geometry(id, xnodes, xcount, 1, 1, 1000000);
-      for (i=0;i<wkt_size;i++)
+      if (cmp_waterway)
       {
-         char *wkt = get_wkt(i);
-         if (strlen(wkt) && (!strncmp(wkt, "POLYGON", strlen("POLYGON")) || !strncmp(wkt, "MULTIPOLYGON", strlen("MULTIPOLYGON"))))
-         {
-             for (place = firstItem(&places); place; place = nextItem(&places, place))
+          wkt_size = build_geometry(id, xnodes, xcount, 1, 1, 1000000);
+          for (i=0;i<wkt_size;i++)
+          {
+             char *wkt = get_wkt(i);
+             if (strlen(wkt) && (!strncmp(wkt, "POLYGON", strlen("POLYGON")) || !strncmp(wkt, "MULTIPOLYGON", strlen("MULTIPOLYGON"))))
              {
-                add_place('R', id, place->key, place->value, &names, &extratags, adminlevel, housenumber, street, addr_place, isin, postcode, countrycode, wkt);
+                 for (place = firstItem(&places); place; place = nextItem(&places, place))
+                 {
+                    add_place('R', id, place->key, place->value, &names, &extratags, adminlevel, housenumber, street, addr_place, isin, postcode, countrycode, wkt);
+                 }
              }
-         }
-         else
-         {
-             /* add_polygon_error('R', id, "boundary", "adminitrative", &names, countrycode, wkt); */
-         }
-         free(wkt);
+             else
+             {
+                 /* add_polygon_error('R', id, "boundary", "adminitrative", &names, countrycode, wkt); */
+             }
+             free(wkt);
+          }
+          clear_wkts();
+      } else {
+          /* waterways result in multilinestrings */
+          char *wkt = get_multiline_geometry(id, xnodes, xcount);
+          if (wkt && strlen(wkt))
+          {
+              for (place = firstItem(&places); place; place = nextItem(&places, place))
+              {
+                 add_place('R', id, place->key, place->value, &names, &extratags, adminlevel, housenumber, street, addr_place, isin, postcode, countrycode, wkt);
+              }
+          }
+          free(wkt);
       }
-      clear_wkts();
 
       for( i=0; i<count; i++ )
       {
