@@ -23,7 +23,7 @@
 */
 
 #ifndef PARSE_O5M_H
-#define PARSE_O5M_h
+#define PARSE_O5M_H
 
 int streamFileO5m(char *filename,int sanitize,struct osmdata_t *osmdata);
 
